@@ -8,6 +8,7 @@ import math
 import json
 import random
 import time
+from enum import Enum
 
 import requests
 from .excs import QQIOError
@@ -58,7 +59,7 @@ class API(object):
     Please http capture request from (mobile) qqmusic mobile web page
     """
 
-    def __init__(self, timeout=1):
+    def __init__(self, timeout=2):
         # TODO: 暂时无脑统一一个 timeout
         # 正确的应该是允许不同接口有不同的超时时间
         self._timeout = timeout
@@ -518,6 +519,33 @@ class API(object):
         js = self.rpc(payload)
         return js['songlist']['data']['tracks']
 
+    def get_diss_info(self, dissid, offset=0, limit=50):
+        """获取歌单的详情
+
+        这是一种特殊的歌单，它的内容是动态的，由官方生成。比如“百万收藏”。
+
+        :param dissid: int, 举个例子 211111 是百万收藏。
+        :return: 参考 fixtures/get_diss_info.json
+        """
+        payload = {
+            'req': {
+                "module": "music.srfDissInfo.aiDissInfo",
+                "method":"uniform_get_Dissinfo",
+                "param": {
+                    "disstid":dissid,
+                    "userinfo":1,  # 不懂啥意思
+                    "tag":1,  # 不懂啥意思
+                    "orderlist":1,
+                    "song_begin": offset,
+                    "song_num": limit,
+                    "onlysonglist":0,
+                    "enc_host_uin":""  # 注：即使登录了，这个也是空
+                }
+            }
+        }
+        js = self.rpc(payload)
+        return js['req']['data']
+
     def get_song_url(self, song_mid):
         uin = self._uin
         songvkey = str(random.random()).replace("0.", "")
@@ -690,5 +718,85 @@ class API(object):
             return 'http://isure.stream.qqmusic.qq.com/{}'.format(midurlinfo[0]['purl'])
         return ''
 
+    class DislikeListType(Enum):
+        singer = 2
+        song = 3
+        _style_unsupported = 4 # TODO: 这是什么？似乎是不喜欢的风格列表，不确定，暂时不支持
+
+    def get_dislike_list(self, page=1, type_=DislikeListType.song, last_id=0):
+        payload = {
+            "req_0": {
+                "module": "music.feedback.FeedbackBlack",
+                "method": "GetDislikeList",
+                "param": {
+                    "Cmd": type_.value,
+                    "Page": page,
+                    "SongLastid": last_id if type_ == API.DislikeListType.song else 0,
+                    "SingersLastid": (
+                        last_id if type_ == API.DislikeListType.singer else 0
+                    ),
+                },
+            },
+        }
+        js = self.rpc(payload)
+        if type_ == API.DislikeListType.song:
+            return js["req_0"]["data"]["Songs"]
+        elif type_ == API.DislikeListType.singer:
+            return js["req_0"]["data"]["Singers"]
+        else:
+            raise QQIOError(f"Unknown dislike list type: {type_}")
+
+    def add_to_dislike_list(self, items, type_=DislikeListType.song):
+        req_param = {
+            "Singers": [],
+            "Songs": [],
+            "Styles": [],
+            "OnlyAdd": 1,
+        }
+        if type_ == API.DislikeListType.song:
+            req_param["Songs"] = items
+        elif type_ == API.DislikeListType.singer:
+            req_param["Singers"] = items
+        else:
+            raise QQIOError(f"Unknown dislike list type: {type_}")
+
+        payload = {
+             "req_0": {
+                 "module": "music.feedback.FeedbackBlack",
+                 "method": "AddDislike",
+                 "param": req_param,
+             },
+        }
+        js = self.rpc(payload)
+        # Response example, {'code': 0, 'data': {'Retcode': 0, 'Msg': '', 'Token': ''}}
+        CodeShouldBe0.check(js['req_0'])
+        return js['req_0']['data']
+
+    def remove_from_dislike_list(self, items, type_=DislikeListType.song):
+        req_param = {
+            "Singers": [],
+            "Songs": [],
+            "Styles": [],
+            "OnlyAdd": 0,
+        }
+        if type_ == API.DislikeListType.song:
+            req_param["Songs"] = items
+        elif type_ == API.DislikeListType.singer:
+            req_param["Singers"] = items
+        else:
+            raise QQIOError(f"Unknown dislike list type: {type_}")
+
+        payload = {
+            "req_0": {
+                "module": "music.feedback.FeedbackBlack",
+                "method": "CancelDislike",
+                "param": req_param,
+            },
+        }
+        js = self.rpc(payload)
+        CodeShouldBe0.check(js)
+        CodeShouldBe0.check(js['req_0'])
+        return js['req_0']['data']
+
 
 api = API()
