@@ -1,5 +1,5 @@
 import logging
-from typing import List, Optional, Protocol
+from typing import List, Optional, Protocol, Tuple
 from feeluown.excs import ModelNotFound
 from feeluown.library import (
     AbstractProvider,
@@ -18,17 +18,26 @@ from feeluown.library import (
     SupportsVideoGet,
     SupportsSongLyric,
     SupportsAlbumGet,
+    SupportsAlbumSongsReader,
     SupportsArtistGet,
     SupportsPlaylistGet,
     SupportsPlaylistSongsReader,
     SupportsRecACollectionOfSongs,
+    SupportsCurrentUserDislikeSongsReader,
+    SupportsCurrentUserDislikeAddSong,
+    SupportsCurrentUserDislikeRemoveSong,
+    SupportsCurrentUserChanged,
     SimpleSearchResult,
     SearchType,
     ModelType,
+    UserModel,
 )
 from feeluown.media import Media, Quality
+from feeluown.utils.dispatch import Signal
 from feeluown.utils.reader import create_reader, SequentialReader
 from .api import API
+from .login import read_cookies
+from .excs import QQIOError
 
 
 logger = logging.getLogger(__name__)
@@ -48,6 +57,10 @@ class Supports(
     SupportsPlaylistGet,
     SupportsPlaylistSongsReader,
     SupportsRecACollectionOfSongs,
+    SupportsAlbumSongsReader,
+    SupportsCurrentUserDislikeSongsReader,
+    SupportsCurrentUserDislikeAddSong,
+    SupportsCurrentUserDislikeRemoveSong,
     Protocol,
 ):
     pass
@@ -65,6 +78,7 @@ class QQProvider(AbstractProvider, ProviderV2):
     def __init__(self):
         super().__init__()
         self.api = API()
+        self.current_user_changed = Signal()
 
     def _(self) -> Supports:
         return self
@@ -77,6 +91,32 @@ class QQProvider(AbstractProvider, ProviderV2):
     def name(self):
         return "QQ 音乐"
 
+    def auto_login(self):
+        cookies = read_cookies()
+        user, err = self.try_get_user_from_cookies(cookies)
+        if user:
+            self.auth(user)
+        else:
+            logger.info(f'Auto login failed: {err}')
+        self.current_user_changed.emit(user)
+
+    def try_get_user_from_cookies(self, cookies) -> Tuple[Optional[UserModel], str]:
+        if not cookies:  # is None or empty
+            return None, 'empty cookies'
+
+        uin = provider.api.get_uin_from_cookies(cookies)
+        if uin is None:
+            return None, "can't extract user info from cookies"
+
+        provider.api.set_cookies(cookies)
+        # try to extract current user
+        try:
+            user = provider.user_get(uin)
+        except QQIOError:
+            provider.api.set_cookies(None)
+            return None, 'get user info with cookies failed, expired cookies?'
+        return user, ''
+
     def use_model_v2(self, mtype):
         return mtype in (
             ModelType.song,
@@ -224,6 +264,10 @@ class QQProvider(AbstractProvider, ProviderV2):
         album = _deserialize(data_album, QQAlbumSchema)
         return album
 
+    def album_create_songs_rd(self, album):
+        album = self.album_get(album.identifier)
+        return create_reader(album.songs)
+
     def user_get(self, identifier):
         data = self.api.user_detail(identifier)
         data["creator"]["fav_pid"] = data["mymusic"][0]["id"]
@@ -253,6 +297,27 @@ class QQProvider(AbstractProvider, ProviderV2):
             pl["logo"] = pl["cover"]
         return [_deserialize(playlist, QQPlaylistSchema) for playlist in playlists]
 
+    def rec_list_daily_songs(self):
+        # TODO: cache API result
+        feed = self.api.get_recommend_feed()
+        card = None
+        for shelf_ in feed['v_shelf']:
+            if 'moduleID' not in shelf_['extra_info']:
+                for batch in shelf_['v_niche']:
+                    for card in batch['v_card']:
+                        if (
+                            card['extra_info'].get('moduleID', '').startswith('recforyou')
+                            and card['jumptype'] == 10014  # 10014->playlist
+                        ):
+                            card = card
+                            break
+        if card is None:
+            logger.warning("No daily songs found")
+            return []
+        playlist_id = card['id']
+        playlist = self.playlist_get(playlist_id)
+        return self.playlist_create_songs_rd(playlist).readall()
+
     def rec_list_daily_playlists(self):
         # TODO: cache API result
         feed = self.api.get_recommend_feed()
@@ -267,14 +332,14 @@ class QQProvider(AbstractProvider, ProviderV2):
         playlists = []
         for batch in shelf['v_niche']:
             for card in batch['v_card']:
-                print(card['title'], card['jumptype'])
                 if card['jumptype'] == 10014:  # 10014->playlist
                     playlists.append(
                         PlaylistModel(identifier=str(card['id']),
                                       source=SOURCE,
                                       name=card['title'],
                                       cover=card['cover'],
-                                      description=card['miscellany']['rcmdtemplate'])
+                                      description=card['miscellany']['rcmdtemplate'],
+                                      play_count=card['cnt'])
                     )
         return playlists
 
@@ -362,6 +427,35 @@ class QQProvider(AbstractProvider, ProviderV2):
         data_songs = self.api.song_similar(int(song.identifier))
         return [_deserialize(data_song, QQSongSchema) for data_song in data_songs]
 
+    def current_user_dislike_create_songs_rd(self):
+        user = self.get_current_user()
+        if user is None:
+            return create_reader([])
+        # FIXME: 如果用户的黑名单歌曲数量较多的话，这样处理则是不够的
+        items = self.api.get_dislike_list(1, API.DislikeListType.song, 0)
+        songs = []
+        for item in items:
+            name = item['Name']
+            title, artists_name = name.split(' - ')
+            song = BriefSongModel(
+                source=SOURCE,
+                identifier=item['ID'],
+                title=title,
+                artists_name=artists_name,
+            )
+            songs.append(song)
+        return create_reader(songs)
+
+    def current_user_dislike_add_song(self, song):
+        items = [{'ID': song.identifier}]
+        js = self.api.add_to_dislike_list(items, API.DislikeListType.song)
+        return js.get('Retcode') == 0
+
+    def current_user_dislike_remove_song(self, song):
+        items = [{'ID': song.identifier}]
+        js = self.api.remove_from_dislike_list(items, API.DislikeListType.song)
+        return js.get('Retcode') == 0
+
 
 def _deserialize(data, schema_cls):
     schema = schema_cls()
@@ -394,26 +488,29 @@ def create_g(func, identifier, schema):
 
 def search(keyword, **kwargs):
     type_ = SearchType.parse(kwargs["type_"])
-    if type_ == SearchType.pl:
-        data = provider.api.search_playlists(keyword)
-        playlists = [_deserialize(playlist, _BriefPlaylistSchema) for playlist in data]
+    type_type_map = {
+        SearchType.so: 0,
+        SearchType.ar: 1,
+        SearchType.al: 2,
+        SearchType.pl: 3,
+        SearchType.vi: 4,
+    }
+    data = provider.api.search(keyword, type_=type_type_map[type_])
+    if type_ == SearchType.so:
+        songs = [_deserialize(song, QQSongSchema) for song in data]
+        return SimpleSearchResult(q=keyword, songs=songs)
+    if type_ == SearchType.ar:
+        artists = [_deserialize(artist, SearchArtistSchema) for artist in data]
+        return SimpleSearchResult(q=keyword, artists=artists)
+    elif type_ == SearchType.al:
+        albums = [_deserialize(album, SearchAlbumSchema) for album in data]
+        return SimpleSearchResult(q=keyword, albums=albums)
+    elif type_ == SearchType.pl:
+        playlists = [_deserialize(playlist, SearchPlaylistSchema) for playlist in data]
         return SimpleSearchResult(q=keyword, playlists=playlists)
-    else:
-        type_type_map = {
-            SearchType.so: 0,
-            SearchType.al: 8,
-            SearchType.ar: 9,
-        }
-        data = provider.api.search(keyword, type_=type_type_map[type_])
-        if type_ == SearchType.so:
-            songs = [_deserialize(song, QQSongSchema) for song in data]
-            return SimpleSearchResult(q=keyword, songs=songs)
-        elif type_ == SearchType.al:
-            albums = [_deserialize(album, _BriefAlbumSchema) for album in data]
-            return SimpleSearchResult(q=keyword, albums=albums)
-        else:
-            artists = [_deserialize(artist, _BriefArtistSchema) for artist in data]
-            return SimpleSearchResult(q=keyword, artists=artists)
+    elif type_ == SearchType.vi:
+        models = [_deserialize(model, SearchMVSchema) for model in data]
+        return SimpleSearchResult(q=keyword, videos=models)
 
 
 provider = QQProvider()
@@ -427,9 +524,12 @@ from .schemas import (  # noqa
     _BriefAlbumSchema,
     _UserArtistSchema,
     _BriefArtistSchema,
-    _BriefPlaylistSchema,
     QQAlbumSchema,
     QQPlaylistSchema,
     QQUserSchema,
     _UserAlbumSchema,
+    SearchAlbumSchema,
+    SearchArtistSchema,
+    SearchPlaylistSchema,
+    SearchMVSchema,
 )  # noqa
